-- ------------------------------------------------------------------------------
-- RP Tags for ElvUI and Total RP 3
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Help: the /rptags command

local RPTAGS     = RPTAGS;
local Const      = RPTAGS.const;
local Utils      = RPTAGS.utils;
local Core       = RPTAGS.core;

local loc        = Utils.locale.loc;
local floc       = Utils.locale.floc;
local hilite     = Utils.text.hilite;

local TAG_PREFIX = Const.TAG;
local fmt        = string.format;

if not Core.helpdata then Core.helpdata = {} end;
local Help = Core.helpdata;


      -- formats and sends a help message, used by the /rptags command -------------------------
local function helpCmd(topic)

      local helpFunctions = {
         ["HELP_PARAM_BLANK"]
           = function()
               return loc("HELPTOPIC_DEFAULT")
               end, -- function
         ["HELP_PARAM_LIST"] 
           = function()
               local h = loc("HELP_LIST_START");
               for k, _ in pairs(Help)
                 do if string.find(string.lower(k), TAG_PREFIX) == 1
                       then h = h .. "[" .. k .. "]  ";
                       end; -- i f
                 end; -- for
               h = h .. loc("HELP_LIST_END");
               return h
               end, -- function
         ["HELP_PARAM_VERSION"]
           = function()
               local GetBuildInfo = GetBuildInfo;
               local GetAddOnMetadata = GetAddOnMetadata;
               local wowBuild, _, _, _ = GetBuildInfo();
       
               return string.format("Version %s by %s ([[[WoW %s]]], [[TRP3 %s]], [[[ElvUI %s]]])", 
                        Const.VERSION, Const.ORAIBI, wowBuild, TRP3_API.globals.version_display, 
                        GetAddOnMetadata("ElvUI", "Version"));
               end, -- function
         ["HELP_PARAM_CHANGES"]
           = function()
               return "[[" .. loc("CHANGES") .. "]]: " .. loc("APP_CHANGES");
               end, -- function
         ["HELP_PARAM_DUMP_SETTINGS"]
           = function()
               local h = "";
               for k, v in pairs(Const.CONFIG.KEYS)
                 do h = h .. "\n[" .. v .. '] = "[[' .. tostring(Utils.config.get(k)) .. ']]"';
                 end; -- for
               return h;
               end, -- function
        } -- help functions

      local h;

      if not topic then topic = "" end;
      local found = false;
      topic = string.lower(topic);
      for k, f in pairs(helpFunctions)
          do if topic == loc(k) then h = f(topic); found = true end;
          end; -- for
      if not found 
         then if     Help[topic]               then h = loc("HELPTOPIC_" .. Help[topic])
              elseif Help[TAG_PREFIX .. topic] then h = loc("HELPTOPIC_" .. Help[TAG_PREFIX .. topic])
              else   h = loc("HELP_ERROR")
              end; -- if/elseif/else
         end; -- if not found

      print( fmt( floc("HELP"), 
                  loc("APP_COLOR_NAME"), 
                  hilite(h) 
           )    );

  end; -- function

-- set up the slash commands
SLASH_RPTAGS1          = loc("APP_SLASH");
SLASH_RPTAGS2          = loc("APP_SLASH_SHORT");
SLASH_RPTAGS3          = loc("APP_SLASH_SHORTER");

SlashCmdList["RPTAGS"] = 
  function(a)
    helpCmd(a);
    end;

-- load the non-tag help topics -------------------------------------------------------------------
--
Help["colors"]     = "COLOR";
Help["contact"]    = "LINKS";
Help["credit"]     = "CREDITS";
Help["credits"]    = "CREDITS";
Help["discord"]    = "LINKS";
Help["download"]   = "LINKS";
Help["elvui"]      = "INTRO";
Help["genders"]    = "GENDER";
Help["guide"]      = "LINKS";
Help["help"]       = "DEFAULT";
Help["intro"]      = "INTRO";
Help["link"]       = "LINKS";
Help["links"]      = "LINKS";
Help["misc"]       = "MISC";
Help["msp"]        = "INTRO";
Help["setting"]    = "SETTINGS";
Help["settings"]   = "SETTINGS";
Help["trp"]        = "INTRO";
Help["trp3"]       = "INTRO";
Help["tutorial"]   = "LINKS";
Help["version"]    = "VERSION";
Help["wiki"]       = "LINKS";
Help["pronouns"]   = "PRONOUNS";

-- topics not yet implemented -----------------------------------------------------------------------
-- Help["recipe"]     = "RECIPES";
-- Help["recipes"]    = "RECIPES";
-- Help["slider"]     = "PSYCH";
-- Help["sliders"]    = "PSYCH";

