-- ------------------------------------------------------------------------------
-- RP Tags for ElvUI and Total RP 3
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Get-Colors: functions to get color values

local RPTAGS           = RPTAGS;
local Const            = RPTAGS.const;
local Utils            = RPTAGS.utils;
local Get              = RPTAGS.get;

if not Get.color then Get.color = {} end;
local Color            = Get.color;

local Config           = Utils.config.get;
local Parse            = Utils.parse; -- the subcategory
local redToGreenHue    = Utils.color.redToGreenHue;
local compareColor     = Utils.color.compare;
local hsvToRgb         = Utils.color.hsvToRgb;
local colorCode        = Utils.color.colorCode;

local getUnitID        = Get.core.unitID;
local getProfile       = Get.core.profile;
local getData          = Get.core.data;
local getField         = Get.core.field;
local profileBlocks    = Get.core.profileBlocks;
local age              = Get.text.age;
local gender           = Get.text.gender;
local chara            = Get.core.chara;

local BIG_PROFILE      = Const.BIG_PROFILE;

      -- get a color string from a specific field on the unit, includes |cff ---------------------------------------
local function getColor(field, u)
      if UnitIsPlayer(u) 
         then local unitID  = getUnitID(u);
              local profile = getProfile(unitID);

              local  fieldValue = getData("characteristics/" .. field, profile); -- get a value
              if not fieldValue then return "" end;

              local  rgb = fieldValue:match("(%x%x%x%x%x%x)");
              if not rgb then return "" end;
              
              return "|cff" .. (rgb)
         else return ""; -- don't change the color
         end; -- if
end   

      -- the simplest color functions
local function getEyeColor(u)  
      return getColor("EH", u);
      end; 

local function getNameColor(u) 
      return getColor("CH", u); 
      end; 

      -- returns a color for the unit based on its gender ----------------------------------------------------
local function getGenderColor(u)
      local _, genderCode = gender(u); 
      return "|cff" .. Config("COLOR_" .. genderCode);
end; 

      -- returns a color for the unit based on its IC/OOC status
local function getStatusColor(u)
      if UnitIsPlayer(u)
         then local status = getField("character/RP", u); -- get the status
              if status == "1" then return "|cff" .. Config("COLOR_IC")  end;
              if status == "2" then return "|cff" .. Config("COLOR_OOC") end;
              return "|cff" .. Config("COLOR_UNKNOWN");
         else return "|cff" .. Config("COLOR_NPC");
         end; -- if
end;  

      -- returns a color based on the unit's age relative to the player
local function getRelativeAgeColor(u)
      if not UnitIsPlayer(u) then return ""; end; -- don't change the color

      if not Config("PARSE_AGE") then return ""; end; -- don't change the color

      local playerAge, parsedPlayer, playerExact = Parse.age(age("player"));
      local unitAge,   parsedUnit,   unitExact   = Parse.age(age(u));

      if not parsedPlayer then return ""; end; -- don't change the color
      if not parsedUnit   then return "|cff" .. Config("COLOR_UNKNOWN"); end;
      
      return compareColor(playerAge, unitAge, playerExact and unitExact);
end;  

      -- returns a color based on the unit's height relative to the player
local function getRelativeHeightColor(u)
      if not UnitIsPlayer(u) then return ""; end; -- don't change the color

      if not Config("PARSE_HW") then return ""; end; -- don't change the color

      local playerHeight, parsedPlayer, playerExact = Parse.height(chara("HE", "player"));
      local unitHeight,   parsedUnit,   unitExact   = Parse.height(chara("HE", u));

      if not parsedPlayer then return ""; end; -- don't change the color
      if not parsedUnit   then return "|cff" .. Config("COLOR_UNKNOWN"); end;
      
      return compareColor(playerHeight, unitHeight, playerExact and unitExact);
end;  

      -- returns a color based on the unit's weight relative to the player
local function getRelativeWeightColor(u)
      if not UnitIsPlayer(u) then return ""; end; -- don't change the color

      if not Config("PARSE_HW") then return ""; end; -- don't change the color

      local playerWeight, parsedPlayer, playerExact = Parse.weight(chara("WE", "player"));
      local unitWeight,   parsedUnit,   unitExact   = Parse.weight(chara("WE", u));

      if not parsedPlayer then return ""; end; -- don't change the color
      if not parsedUnit   then return "|cff" .. Config("COLOR_UNKNOWN"); end;
      
      return compareColor(playerWeight, unitWeight, playerExact and unitExact);
end;  

      -- hilite colors
local function getHiliteColor(n)
      if not n then n = 1; end;
      return "|cff" .. Config("COLOR_HILITE_" .. n);
end; 

      -- returns a color based on the size of the unit's "about" field
local function computeProfileSizeColor(u)
      local size = profileBlocks(u);
      if   size == -1 then return Config("COLOR_NEUTRAL"); end;
      local  hue = redToGreenHue(size, 0, BIG_PROFILE, true);
      local  r, g, b = hsvToRgb(hue, 1, 1, 1);
      return colorCode(r, g, b);
end; 

-- functions available under RPTAGS.get.color
--
Color.age         = getRelativeAgeColor;
Color.custom      = getColor;
Color.eye         = getEyeColor;
Color.gender      = getGenderColor;
Color.height      = getRelativeHeightColor;
Color.hilite      = getHiliteColor;
Color.name        = getNameColor;
Color.profileSize = computeProfileSizeColor;
Color.status      = getStatusColor;
Color.weight      = getRelativeWeightColor;
