-- ------------------------------------------------------------------------------
-- RP Tags for ElvUI and Total RP 3
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Get-Core: core functions for getting data from trp3
  --
  -- function list at end

local RPTAGS           = RPTAGS;
local Const            = RPTAGS.const;
local Utils            = RPTAGS.utils;
local Get              = RPTAGS.get;

if not Get.core then Get.core = {}; end;
local Core = Get.core;

local Config      = Utils.config.get;
local Format      = Utils.format; -- the subcategory
local getData     = Utils.import.trp3.getData;
local getProfile  = Utils.import.trp3.getProfile;
local getUnitID   = Utils.import.trp3.getUnitID;
local profileSize = Utils.import.trp3.profileSize;
local sanitize    = Utils.text.sanitize;
local wrap        = Utils.text.wrap;

local BIG_PROFILE = Const.BIG_PROFILE;

--
      -- primary method of retrieving TRP3 data --------------------------------------------------------------------
local function getField(fieldName, u, sizecode)
      if not u then return "" end;

      -- ---------------------------------------------------------------------------------------
      if UnitIsPlayer(u)
         then local  unitID     = getUnitID(u);
              local  profile    = getProfile(unitID);
              local  fieldValue = getData(fieldName, profile);
                     if fieldValue == nil then return "" end;
                     fieldValue = sanitize(tostring(fieldValue));
              return wrap(fieldValue, sizecode);

         else return ""

         end  -- if player
end  

      -- alias to get characteristics ------------------------------------------------------------------------------
local function getCharacteristic(partial_field, u, sizecode)
      return getField("characteristics/" .. partial_field, u, sizecode)
end 

      -- retrieve "miscellaneous" fields such as MOTTO
local function scanThroughArbitraryFields(fieldName, u, sizecode)

      if UnitIsPlayer(u)
         then local  unitID     = getUnitID(u);
              local  profile    = getProfile(unitID);
              if not profile then return "", false; end;
              if not profile.characteristics then return "", false; end;
              if not profile.characteristics.MI then return "", false; end;
              for _, t in pairs(profile.characteristics.MI) do
                     if string.find(string.lower(t["NA"]), fieldName) == 1
                        then -- found a match!
                                return wrap(t["VA"], sizecode), true;
                                end; -- if
                     end -- for
              return "", false;
          else 
              return "", false;
          end; -- if                 
end; 

      -- the number of trp3 blocks needed to transmit a profile's "about" section
local function getProfileSizeNum(u) 
   local s = -1;
   if UnitIsPlayer(u) 
      then local unitID  = getUnitID(u);
           local profile = getProfile(unitID);
           if profile and profile.about then s = profileSize(profile.about); end;
      end; -- if
    return s;
end; 

      -- returns a formatted version of getProfileSizeNum
local function getProfileSizeTag(u)
      if UnitIsPlayer(u)
         then local size = getProfileSizeNum(u);
              if size == -1 then return ""; end;
              return Format.sizewords(size, BIG_PROFILE, Config("PROFILESIZE_FMT"));
         else return "";
         end;
end; 

-- functions available through RPTAGS.get.core
--
Core.chara         = getCharacteristic;
Core.data          = Utils.import.trp3.getData;
Core.field         = getField;
Core.misc          = scanThroughArbitraryFields;
Core.profile       = Utils.import.trp3.getProfile;
Core.profileBlocks = getProfileSizeNum;
Core.profileSize   = Utils.import.trp3.profileSize;
Core.profileSize   = getProfileSizeTag;
Core.trpName       = Utils.import.trp3.getTrpName;
Core.unitID        = Utils.import.trp3.getUnitID;
