-- ------------------------------------------------------------------------------
-- RP Tags for ElvUI and Total RP 3
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Get

local RPTAGS           = RPTAGS;

local Const            = RPTAGS.const;
local Utils            = RPTAGS.utils;
local Get              = RPTAGS.get;

if not Get.text then Get.text = {}; end;
local Text = Get.text;

local Config           = Utils.config.get;
local fmt              = string.format;
local loc              = Utils.locale.loc;
local floc             = Utils.locale.floc;
local Format           = Utils.format; -- the subcategory
local Parse            = Utils.parse; -- the subcategory
local wrap             = Utils.text.wrap;
local split            = Utils.text.split;
local tc               = Utils.text.titlecase;
local lc               = string.lower;
local uc               = string.upper;

local getTrpName       = Get.core.trpName;
local field            = Get.core.field;
local chara            = Get.core.chara;
local misc             = Get.core.misc;
local getUnitID        = Get.core.unitID;
local getProfile       = Get.core.profile;
local getData          = Get.core.data;


      -- functions that just get and return straight values
local function getAge(u)      
      return chara("AG", u, "MISC"); 
      end; 

local function getBP(u)
      return chara("BP", u, "MISC"); 
      end; 

local function getCurr(u) 
      return field("character/CU", u, "CURR"); 
      end; 

local function getEyes(u) 
      return chara("EC", u, "MISC"); 
      end; 

local function getFT(u) 
      return chara("FT", u, "TITLE"); 
      end; 

local function getHome(u) 
      return chara("RE", u, "MISC");
      end; 

local function getInfo(u) 
      return field("character/CO", u, "CURR"); 
      end; 

local function getLastName(u) 
      return chara("LN", u, "HALFNAME");
      end; 

local function getTitle(u) 
      return chara("TI", u, "HALFTITLE");
      end; 

      -- retrieve at-first-glance fields ---------------------------------------------------------------------------
local function getGlance(slot, u, showTitle, showText, showIcon) -- , delimiter, separator)
      if not UnitIsPlayer(u) then return ""; end; -- if

      local delimiter = Config("GLANCE_DELIM");
      local separator = Config("GLANCE_COLON");

      if slot == 0
         then local  allGlances = {};
              for i =  1, 5 
                do local iGlance =  getGlance(i, u, showTitle, showText, showIcon);
                   if iGlance ~= "" then table.insert(allGlances, iGlance) end;
                end -- for
              return table.concat(allGlances, delimiter or "");
         else local  path    = "misc/PE/" .. slot;
              local  unitID  = getUnitID(u);
              local  profile = getProfile(unitID);

              local  active = getData(path .. "/AC", profile);
              if not active then return "" end;

              local  title = field(path .. "/TI", u, "MISC");
              local  text =  field(path .. "/TX", u, "GLANCE");
              local  icon =  field(path .. "/IC", u, "UNLIM");

              local value = "";

              if icon  and showIcon
                 then value = value .. "|TInterface\\ICONS\\" .. icon .. ":0|t "; end;
              if title and showTitle                       
                 then value = value .. title; end;
              if title and showTitle and text and showText 
                 then value = value .. separator; end;
              if text  and showText                        
                 then value = value .. text; end;
              
              if value == "" and showTitle then value = "..."; end;
              return value;

         end  -- if
end   

      -- whether the unit has any buffs affecting their scale -----------------------------------------------
local function getSizeBuff(unit)
      local sizeBuff = nil;
      if UnitIsPlayer(unit)
         then for buffID,_ in pairs(Const.SIZEBUFFS)
                do for i = 1,40
                   do local _, _, _, _, _, _, _, _, _, spellID = UnitAura(unit,i);
                      if spellID == buffID then sizeBuff = buffID; end;
                   end; -- for i = 1,40
                end; -- for buffID
              if sizeBuff -- at this point either we found a buff or we haven't
                 then return Format.sizebuff(sizeBuff, 
                                 Config("SIZEBUFF_FMT"), 
                                 UnitRace(unit) == "PANDAREN"); 
                 else return "";
                 end; -- if sizeBuff
         else return "";
         end; -- if playser
end   

      -- get the height of a player --------------------------------------------------------------------------------
local function getHeight(u) 
      if not UnitIsPlayer(u) then return ""; end; -- if

      local height = chara("HE", u, "UNLIM"); 
      if    height == "" then return ""; end; -- if

      local cm, s;
      if Config("PARSE_HW") 
         then cm, s = Parse.height(height)  -- we don't care about the exactness
         end; -- if

      if s -- if we parsed it and got a cm result
         then return Format.cm(cm,     Config("UNITS_HEIGHT"));
         else return Format.cm(height, Config("UNITS_HEIGHT"));
         end; -- if
end   

      -- get the weight of a player --------------------------------------------------------------------------------
local function getWeight(u) 
      if not UnitIsPlayer(u) then return ""; end; -- if

      local weight = chara("WE", u, "UNLIM"); 
      if    weight == "" then return ""; end; -- if

      local kg, s;
      if Config("PARSE_HW")
         then kg, s = Parse.weight(weight)
         end; --  if
         
      if s -- we parsed it and got a kg result
         then return Format.kg(kg,     Config("UNITS_WEIGHT")); 
         else return Format.kg(weight, Config("UNITS_WEIGHT")); 
         end; -- if

end   

      -- get the race of a player or other unit --------------------------------------------------------------------
local function getRace(u)
      local trpVersion = chara("RA", u, "MISC");
      local systemRace = wrap(UnitRace(u), "MISC");
      if string.len(trpVersion) > 0 
         then return trpVersion 
         else return systemRace
         end  -- if trpversion
end   

      -- get the class of a player or another unit -----------------------------------------------------------------
local function getClass(u)
      local trpVersion     = chara("CL", u, "MISC");
      local systemClass, _ = Utils.text.titlecase(UnitClassBase(u));
      systemClass = wrap(systemClass, "MISC");
      if string.len(trpVersion) > 0
         then return trpVersion
         else return systemClass
         end  -- if player
end   

      -- get the name of a player or other unit ---------------------------
local function getName(u)

      local  sysName = UnitName(u);
      local  trpName = getTrpName(u) or "";

      if string.len(trpName) > 0
         then return wrap(trpName, "NAME");
         else return wrap(sysName, "NAME");
         end; --if 
end   

      -- get the first name of a player or other unit ---------------------------
local function getFirstName(u)
      local firstName;

      local sysName      = wrap(UnitName(u),   "HALFNAME");
      local trpFirstName = chara("FN", u,      "HALFNAME");
      local trpFullName  = wrap(getTrpName(u), "HALFNAME");

      if        string.len(trpFirstName) > 0 then firstName = trpFirstName;
         elseif string.len(trpFullName)  > 0
                then local fn = string.match(trpFullName, "^(%S+)");
                     if fn then firstName = fn else firstName = trpFullName end;
                else local fn = string.match(sysName,     "^(%S+)");
                     if fn then firstName = fn else firstName = sysName;    end;
         end; -- if

      return firstName;
end   

      -- years, an age function that returns a number of years
local function getYears(u, flag)
      local age = chara("AG", u, "UNLIM");

      if age == "" then return ""; end;

      local years, s, x = Parse.age(age);
      age = wrap(age, "MISC");

      if not s then return age; end;
      if not x then years = loc("ABOUT") .. " " .. years end;
      years = years .. " " .. loc("AGE_YEARS");
      if flag then years = years .. " " .. loc("YEARS_" .. string.upper(flag)) end;
      return years;
end; 


      -- ic/ooc status
local function getStatus(u)
      if UnitIsPlayer(u)
         then local status =  field("character/RP", u, "UNLIM"); -- get the status
              if status == "1" 
                then return loc("IS_IC")
                else return loc("IS_OOC")
                end;

         else return loc("IS_NPC");
         end; -- if isplayer
end   


local function getNpcStatus(u)
      if not UnitIsPlayer(u) 
         then return loc("IS_NPC") 
         else return "" 
         end;
end; 


local function getIcStatus(u)
      local status = field("character/RP", u, "UNLIM"); -- get the status
      if status == "1" 
         then return loc("IS_IC") 
         else return ""
         end;
end; 
      
local function getOocStatus(u)
      local status = field("character/RP", u, "UNLIM"); -- get the status
      if status == "2" 
         then return loc("IS_OOC") 
         else return ""
         end;
end; 
      
local function getGender(u)
      local genderField = misc("gender", u, "MISC");
      local race        = getRace(u);
      local gameSex     = UnitSex(u);

      if Config("PARSE_GENDER") 
         then local knownGender, s;

              if string.len(genderField) > 0 
                 then knownGender, s = Parse.gender(genderField);
                      if s then return genderField, knownGender 
                           else return genderField, "UNKNOWN"; end;
                 end; -- if genderfield

         knownGender, s = Parse.gender(string.gsub(race, "[Hh]uman", "#####"));
         if s then return loc("GENDER_" .. knownGender), knownGender; end;
      end; -- if parse gender

      -- so now we'll try what the game thinks they are
      if gameSex == 3 then return loc("GENDER_FEMALE"), "FEMALE"; end;
      if gameSex == 2 then return loc("GENDER_MALE"),   "MALE";   end;
      if gameSex == 1 then return loc("GENDER_NEUTER"), "NEUTER"; end;

      -- fallback on "heck i dunno"
      return loc("GENDER_UNKNOWN"), "UNKNOWN";

end; 

local function getPronoun(u, pro)
      -- load the localized pronoun data -------------------------------------------------------------------------------
      local pronounList = loc("PRONOUN_TAGS");
      local genderList  = loc("GENDERS") .. ";" .. loc("EXTRA_PRONOUN_SETS");

      -- check the input parameters ------------------------------------------------------------------------------------
      if not pro or not u                       then return "", nil end; -- if we don't have both params
      if not string.len(pro) == 1               then return "", nil end; -- if code for pronoun isn't 1 char
      if not string.match(pronounList, lc(pro)) then return "", nil end; -- if code isn't on the pronoun list

      local function case(s)  -- helper function -----------------------------------------------------------------------
            if string.match(pro, "%u") 
               then return tc(s) 
               else return lc(s) 
               end -- if
            end;

      local function pronoun(g, v) -- helper function ------------------------------------------------------------------
            return loc("PRONOUN_" .. uc(g) .. "_" .. uc(v));
            end;

      local function allPronouns(g)
            local all = "";
            for t in string.gmatch(pronounList, "%a")
                do all = all .. "/" .. pronoun(g, t) end;
            return all:sub(2);
            end;
            
      local pronounsField = misc("pronouns", u); -- get the pronouns field ---------------------------------------------
      local pronounIndex  = string.find(lc(pronounList), lc(pro));

      -- debuggery -----------------------------------------------------------------------------------------------------
      -- print("Pronouns field: [" .. pronounsField .. "]");
      -- print("Gender list: ["    .. genderList ..    "]");
      -- print("Pronoun tag: ["    .. pro ..           "]");


      if pronounsField ~= ""  
         then -- they have a custom pronouns field ---------------------------------------------------------------------

              local customPronouns = split(pronounsField, "/");
              for _, g in ipairs(split(genderList, ";")) 
                  do -- loop g == loop through the genders -------------------------------------------------------------
                     if lc(pronounsField) == lc(g) -- Does this gender's name match the gender field? ------------------
                        then return case(pronoun(g, pro)), allPronouns(g); end; -- if

                     for _, c in ipairs(customPronouns)
                         do -- loop c == loop through their customs ----------------------------------------------------
                            for t in string.gmatch(pronounList, "%a") 
                                do -- loop t == loop through the tags --------------------------------------------------
                                   if lc(pronoun(g, t)) == lc(c) -- found a match, so we'll assume this is our gender
                                      then return case(pronoun(g, pro)), allPronouns(g); 
                                      end -- if
                                   end -- loop t
                          end -- loop c

                  end -- loop g

              -- did not find a match on any of the sets but they still have a custom field ----------------------------
              --
              local p = customPronouns[pronounIndex];
              if p then return case(p),                 pronounsField;
                   else return case(customPronouns[1]), pronounsField;
                   end -- if

         else -- they don't have a pronouns field ----------------------------------------------------------------------

              local _, genderMatch = getGender(u);
              local p = pronoun(genderMatch, lc(pro));
              if p ~= "" then return case(p), allPronouns(genderMatch)
                         else return "",      ""
                         end; -- if 

         end; -- if pronounsField --------------------------------------------------------------------------------------
end;

local function getAllPronouns(u)
      local s = loc("PRONOUN_TAGS"):sub(1,1);
      local _, all = getPronoun(u, s);
      return all;
end;

local function getMiscAndTweakOutput(miscField, u)
  
  local Tweaks = { 
        ["motto"]    = function(unit) local  m, f = misc("motto", unit);
                                      return f and fmt(floc("MOTTO"), m) or "" end,

        ["house"]    = function(unit) local  h, hf = misc("house", unit);
                                      local  f, ff = misc("family", unit);
                                      return hf and h or ff and f or misc("tribe") end,

        ["family"]   = function(unit) local  f, ff = misc("family", unit);
                                      local  h, hf = misc("house", unit);
                                      return ff and  f or hf and h or misc("tribe") end,

        ["tribe"]    = function(unit) local  t, tf = misc("tribe", unit);
                                      local  f, ff = misc("family", unit);
                                      return tf and  t or ff and f or misc("house") end,

        ["markings"] = function(unit) local  m, f =  misc("markings", unit);
                                      return f  and  m or misc("tattoos", unit) end,

        ["religion"] = function(unit) local  r, f =  misc("religion", unit);
                                      return f  and  r or misc("faith", unit) end,

        ["rstatus"]  = function(unit) local  r, rf = misc("relationship status", unit);
                                      local  m, mf = misc("marital status", unit);
                                      return rf and  r or mf and m or misc("romantic status", unit) end,
     };

  if Tweaks[miscField] 
     then return Tweaks[miscField](u) 
     else return misc(miscField, u) 
     end; -- if
end;

-- functions available under RPTAGS.get.text

Text.age        = getAge;
Text.birthplace = getBP;
Text.class      = getClass;
Text.curr       = getCurr;
Text.eyes       = getEyes;
Text.firstname  = getFirstName;
Text.fulltitle  = getFT;
Text.gender     = getGender;
Text.glance     = getGlance;
Text.height     = getHeight;
Text.home       = getHome;
Text.ic         = getIcStatus;
Text.info       = getInfo;
Text.lastname   = getLastName;
Text.misc       = getMiscAndTweakOutput;
Text.name       = getName;
Text.npc        = getNpcStatus;
Text.ooc        = getOocStatus;
Text.race       = getRace;
Text.sizebuff   = getSizeBuff;
Text.status     = getStatus;
Text.title      = getTitle;
Text.weight     = getWeight;
Text.years      = getYears;
Text.pronoun    = getPronoun;
Text.pronouns   = getAllPronouns;
