-- ------------------------------------------------------------------------------
-- RP Tags for ElvUI and Total RP 3
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.

--Notices:

--You do not have to comply with the license for elements of the material
--in the public domain or where your use is permitted by an applicable
--exception or limitation.

--No warranties are given. The license may not give you all of the
--permissions necessary for your intended use. For example, other rights
--such as publicity, privacy, or moral rights may limit how you use the material.
-- ------------------------------------------------------------------------------

local RPTAGS     = RPTAGS;
local Utils      = RPTAGS.utils;
local TRP3_API   = TRP3_API;

local fmt        = string.format;

local createPage   = Utils.config.setup;
local loc          = Utils.locale.loc;
local floc         = Utils.locale.floc;
local Register     = Utils.import.trp3.register;
local hi           = Utils.text.hilite;
local refreshFrame = Utils.tags.refreshFrame;
-- local refreshAll   = Utils.tags.refreshAll;
local Config       = Utils.config.get;
local getUnitID    = Utils.import.trp3.getUnitID; -- can't use get because trp3 loads before get
local unitIDToInfo = Utils.import.trp3.unitIDToInfo;
local UnitInRaid   = UnitInRaid;
local UnitInParty  = UnitInParty;

local EVENT_LOADED       = TRP3_API.events.WORKFLOW_ON_LOADED;
local EVENT_NEWDATA      = TRP3_API.events.REGISTER_DATA_UPDATED;

Register.addon( {
  ["name"]        = fmt(floc("APP_NAME"), loc("APP_NAME"), loc("APP_VERSION"), loc("APP_CREATOR")),
  ["description"] = loc("APP_DESC"),
  ["version"]     = loc("APP_VERSION"),
  ["id"]          = loc("APP_ID"),
  ["minVersion"]  = 3,
  ["autoEnable"]  = true,
  ["onStart"]     = function() 
    Register.event(EVENT_LOADED, -- on start, we want to register an event for when it's done loading
      function() -- this is the startup function
        local configPages, s = createPage();
    
        -- now we register the structure we just defined
        if s then for _, page in ipairs(configPages)
                      do Register.config.page(page); 
                      end; -- for
             end; -- if successful
    
        -- we're going to listen for trp3 update event and make elvui update when it happens
        --
        Register.callback(EVENT_NEWDATA, function(unitID)
           if not unitID then return end
           
           local unit_name, _ = unitIDToInfo(unitID);

--           print("Ora, callback parameter is " .. tostring(unitID));
--           print(string.format("Ora, %s=%s, %s=%s", "unit_name", unit_name, "unit_realm", unit_realm));

--            print(string.format( "Ora, unitIDs are %s=%s, %s=%s, %s=%s, %s=%s", 
--                                 "target",       getUnitID("target")       or "",
--                                 "player",       getUnitID("player")       or "",
--                                 "focus",        getUnitID("focus")        or "",
--                                 "targettarget", getUnitID("targettarget") or ""
--                    ));

--           print(string.format("Ora, %s=%s, %s=%s", 
--                               "UnitInRaid",  UnitInRaid(unitID),
--                               "UnitInParty", UnitInParty(unitID),
--                               "beep", "beep"
--                )             );
--
           if        getUnitID("target")       == unitID then refreshFrame("target") 
              elseif getUnitID("player")       == unitID then refreshFrame("player")
              elseif getUnitID("focus")        == unitID then refreshFrame("focus")
              elseif getUnitID("targettarget") == unitID then refreshFrame("targettarget")
              elseif UnitInRaid(unitID)                  then refreshFrame("raid")
              elseif UnitInRaid(unit_name)               then refreshFrame("raid")
              elseif UnitInParty(unitID)                 then refreshFrame("party")
              elseif UnitInParty(unit_name)              then refreshFrame("party")
              end; -- if

           end -- end of our callback handler for new data
        );
    
        -- if the user wants the login message, print it
        if Config("LOGIN_MESSAGE") 
           then print(fmt(floc("APP_LOAD"), loc("APP_NAME"), loc("APP_VERSION"),
                          loc("APP_CREATOR"), loc("APP_SLASH")));
           end; -- if
    
        if Config("CHANGES_MESSAGE")
           then print(hi(loc("APP_COLOR_NAME")  .. " " ..  loc("APP_VERSION") .. " [["  .. 
                             loc("CHANGES")     .. "]]: " ..  loc("APP_CHANGES")));
           end; -- if 

      end -- of the initialization function which is waiting for the loading

      ); -- closes the register event params

    end -- of the onStart function

}); -- closes the addon registering table and the function params
    
