-- ------------------------------------------------------------------------------
-- RP Tags for ElvUI and Total RP 3
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Parse: Utilities for analzying a string and converting it to a value for formatting

local RPTAGS     = RPTAGS;
local Utils      = RPTAGS.utils;

if not Utils.parse then Utils.parse = {}; end;
local Parse      = Utils.parse;

local loc        = Utils.locale.loc;
local isnum      = Utils.text.isnum;
local match      = Utils.text.match;
local split      = Utils.text.split;
local Config     = Utils.config.get;
local rot13      = Utils.text.rot13;

-- ---------------------------------------------------------------------------------------------------------------------
      -- returns a numeric value, a boolean indicating if parsed, and a boolean indicating if exact value
local function parseAge(s, race)

      local age, success, exact = 0, false, false;
      local num = tonumber;
      
      s = string.gsub(s, ",", ""); -- so we can say "5,000 years"

      local ish    = string.match(s, "(%d+)ish");
      local years  = string.match(s, "(%d+) years");
      local cent   = string.match(s, "(%d+) centuries");
      local mill   = string.match(s, "([%d.]+) millen+ia");
      local dec    = string.match(s, "(%d+)s");
      local number = string.match(s, "(%d+)");

      if       isnum(s)      then age = num(s);           success = true; exact = true;
        elseif isnum(ish)    then age = num(ish);         success = true; exact = false; 
        elseif isnum(years)  then age = num(years);       success = true; exact = true;
        elseif isnum(cent)   then age = num(cent) * 100;  success = true; exact = false;
        elseif isnum(mill)   then age = num(mill) * 1000; success = true; exact = false;
        elseif isnum(dec)    then age = num(dec) + 5;     success = true; exact = false;
        elseif isnum(number) then age = num(number);      success = true; exact = true;
      end; -- if

      if match(s, loc("ABOUT")) then exact = false; end;
      if match(s, race)         then exact = false; end;

      return age, success, exact;
end;  

-- ---------------------------------------------------------------------------------------------------------------------
      -- returns a numeric value, a boolean indicating if parsed, and a boolean indicating if exact value
local function parseHeight(s)

      local height, success, exact = 0, false, false;
      local num = tonumber;

      local cm         = string.match(s, "([%d.]+) cm"      );
      local meters     = string.match(s, "([%d.]+) meter"  );
      local fift, fiin = string.match(s, "(%d+)' *(%d+)"    );
      local feet       = string.match(s, "(%d+) feet"      );
      local foft, foin = string.match(s, "(%d+) foot (%d+)");

      if       isnum(s)      then height = num(s);                               success = true; exact = true; 
        elseif isnum(cm)     then height = num(cm);                              success = true; exact = true;
        elseif isnum(meters) then height = num(meters) * 100;                    success = true; exact = true; 
        elseif isnum(fiin)   then height = 30.48 * num(fift) + 2.54 * num(fiin); success = true; exact = true;
        elseif isnum(feet)   then height = 30.48 * num(feet);                    success = true; exact = false;
        elseif isnum(foin)   then height = 30.48 * num(foft) + 2.54 * num(foin); success = true; exact = false;
        end; -- if

      if match(s, loc("ABOUT")) then exact = false; end;

      return math.floor(height + 0.5), success, exact;
end;  

-- ---------------------------------------------------------------------------------------------------------------------
      -- returns a numeric value, a boolean indicating if parsed, and a boolean indicating if exact value
local function parseWeight(s)
      local weight, success, exact = 0, false, false;
      local num = tonumber;

      local lb = string.match(s, "(%d+) lb");
      local pounds = string.match(s, "(%d+) pound");
      local kg = string.match(s, "([%d.]+) kg");
      local kilo = string.match(s, "([%d.]+) kilo");

      if       isnum(s)  then weight = num(s);                 success = true; exact = true; 
        elseif isnum(lb) then weight = num(lb) * 0.45;         success = true; exact = true;
        elseif isnum(pounds) then weight = num(pounds) * 0.45; success = true; exact = true;
        elseif isnum(kg) then weight = num(kg);                success = true; exact = true;
        elseif isnum(kilo) then weight = num(kilo);            success = true; exact = true;
        end; --if

      if match(s, loc("ABOUT")) then exact = false; end;

      return weight, success, exact;
end;  

-- ---------------------------------------------------------------------------------------------------------------------
local function parseGenderString(s)
      local genderList = split(loc("GENDERS"),";");

      for _, t in ipairs(genderList)
        do local genderAliases = loc("GENDER_ALIASES_"  .. t);
           if Config("ADULT_GENDERS") 
              then genderAliases = genderAliases .. ";" .. rot13(loc("GENDER_ADULT_" .. t))
              end; -- if
           local aliasList = split(genderAliases,";");

           for _, a in ipairs(aliasList)
             do if match(s, a) then return t, true; end; -- if
             end; -- for
        end; -- for
      return "UNKNOWN", false;
end; 

-- Functions available under RPTAGS.utils.parse
--
Parse.gender = parseGenderString;
Parse.age    = parseAge;
Parse.weight = parseWeight;
Parse.height = parseHeight;
