-- ------------------------------------------------------------------------------
-- RP Tags for ElvUI and Total RP 3
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Config: Utilities for reading and registering configuration values

local RPTAGS     = RPTAGS;
local Const      = RPTAGS.const;
local Utils      = RPTAGS.utils;

if not Utils.config then Utils.config = {} end;
local Config = Utils.config;

local loc         = Utils.locale.loc;
local hi          = Utils.text.hilite;
local refreshAll  = Utils.tags.refreshAll;
local Format      = Utils.format;                 -- the subcategory
local Register    = Utils.import.trp3.register.config;

local Keys        = Const.CONFIG.KEYS;
local Default     = Const.CONFIG.DEFAULTS;
local BIG_PROFILE = Const.BIG_PROFILE;
local H1          = "TRP3_ConfigH1";
local CHECK       = "TRP3_ConfigCheck";
local SELECT      = "TRP3_ConfigDropDown";
local COLOR       = "TRP3_ConfigColorPicker";
local BUTTON      = "TRP3_ConfigButton";
local SLIDER      = "TRP3_ConfigSlider";
local DEPS        = "dependentOnOptions"; 

      -- load our configuration settings and defaults into trp3
local function setUpConfig()
  for k, v in pairs(Keys)
      do Register.key(v, Default[k]);
         Register.handler(v, refreshAll);
      end -- for

-- these set up the pulldown menus for the config page
  local    weights,     heights, sizebuffs, profilesizes, _;
        _, weights      = Format.kg(73);
        _, heights      = Format.cm(168);
        _, sizebuffs    = Format.sizebuff(8212);
        _, profilesizes = Format.sizewords(7, BIG_PROFILE);

-- used below
  local TRUNC_OR_WRAP = {
      { loc("OPT_LINE_TRUNCATE"),     "TRUNCATE",     },
      { loc("OPT_LINE_WORD_TRUNC"),   "WORD_TRUNC",   },
      { loc("OPT_LINE_WRAP_ONCE"),    "WRAP_ONCE",    },
      { loc("OPT_LINE_WRAP_TWICE"),   "WRAP_TWICE",   },
      { loc("OPT_LINE_WRAP_FOREVER"), "WRAP_FOREVER", },
    };

-- this works with the above to finish setting up the pulldowns
  local LIST = { -- yeah there are probably easier ways to do this but whatever
    UNITS_HEIGHT = { 
      { heights["CM"],              "CM" },
      { heights["CM_FT_IN"],        "CM_FT_IN" },
      { heights["CM_IN"],           "CM_IN" },
      { heights["FT_IN"],           "FT_IN" },
      { heights["FT_IN_CM"],        "FT_IN_CM" },
      { heights["IN"],              "IN" }, 
      { heights["IN_CM"],           "IN_CM" },
      { heights["HEIGHT_PASSTHRU"], "HEIGHT_PASSTHRU" },
     },
    UNITS_WEIGHT = { 
      { weights["KG"],              "KG" },
      { weights["KG_LB"],           "KG_LB" },
      { weights["KG_ST_LB"],        "KG_ST_LB" },
      { weights["LB_KG"],           "LB_KG",}, 
      { weights["LB_ST"],           "LB_ST",},
      { weights["ST_LB"],           "ST_LB",},
      { weights["ST_LB_KG"],        "ST_LB_KG",},
      { weights["ST_LB_LB_KG"],     "ST_LB_LB_KG" },
      { weights["WEIGHT_PASSTHRU"], "WEIGHT_PASSTHRU" },
     },
    PROFILESIZE_FMT = { 
      { profilesizes["WRD_BRC_NUM"], "WRD_BRC_NUM" },
      { profilesizes["WRD_PAR_NUM"], "WRD_PAR_NUM" },
      { profilesizes["BRC_WRD_NUM"], "BRC_WRD_NUM" },
      { profilesizes["PAR_WRD"],     "PAR_WRD" },
      { profilesizes["WRD"],         "WRD" },
      { profilesizes["BRC_LTR_NUM"], "BRC_LTR_NUM" }, 
      { profilesizes["BRC_LTR"],     "BRC_LTR" }, 
      { profilesizes["LTR"],         "LTR",},
      { profilesizes["BRC_NUM"],     "BRC_NUM",},
      { profilesizes["NUM"],         "NUM",},
     },
    SIZEBUFF_FMT = { 
      { sizebuffs["PCT"],          "PCT" },
      { sizebuffs["PCT_BRC"],      "PCT_BRC" },
      { sizebuffs["PCT_BUFF"],     "PCT_BUFF" },
      { sizebuffs["BUFF_PCT"],     "BUFF_PCT" },
      { sizebuffs["BUFF"],         "BUFF" },
      { sizebuffs["BUFF_BRC"],     "BUFF_BRC" },
      { sizebuffs["BUFF_PCT_BRC"], "BUFF_PCT_BRC" },
    },
    GLANCE_COLON = { 
      { loc("CONFIG_GLANCE_COLON_COLON"),   ": "  , },
      { loc("CONFIG_GLANCE_COLON_COMMAS"), ", "  , },
      { loc("CONFIG_GLANCE_COLON_DASHES"),  " - " , },
      { loc("CONFIG_GLANCE_COLON_SLASHES"), " / " , },
      { loc("CONFIG_GLANCE_COLON_SPACES"),  " "   , }, 
    },
    GLANCE_DELIM = { 
      { loc("CONFIG_GLANCE_DELIM_COMMAS"),     ", ",  },
      { loc("CONFIG_GLANCE_DELIM_NEWLINES"),   "\n",  },
      { loc("CONFIG_GLANCE_DELIM_SEMICOLONS"), "; ",  },
      { loc("CONFIG_GLANCE_DELIM_SLASHES"),    " / ", },
      { loc("CONFIG_GLANCE_DELIM_SPACES"),     ", ",  },
    },
    NAME_WRAP   = TRUNC_OR_WRAP,
    TITLE_WRAP  = TRUNC_OR_WRAP,
    GLANCE_WRAP = TRUNC_OR_WRAP,
    MISC_WRAP   = TRUNC_OR_WRAP,
    CURR_WRAP   = TRUNC_OR_WRAP,
   }; -- closes the LIST table

   -- these helper functions define various types of trp3 help screen widgets
  local function h1(key)
    return { 
      inherit     = H1,
      title       = loc("CONFIG_H1_" .. key) }; 
    end; -- h1()

  local function check(key, dep)
    return { 
      inherit     = CHECK,
      title       = hi(loc("CONFIG_" .. key)),
      configKey   = Keys[key],
      help        = hi(loc("CONFIG_" .. key .. "_TT")),
      [DEPS]      = dep }; 
    end; -- check()

  local function list(key, dep)
    return { 
      inherit     = SELECT,
      title       = hi(loc("CONFIG_" .. key)),
      configKey   = Keys[key],
      listContent = LIST[key], 
      help        = hi(loc("CONFIG_" .. key .. "_TT")),
      [DEPS]      = dep }; 
    end; -- list()

  local function size(key, dep)
      return {
        inherit   = SLIDER,
        title     = hi(loc("CONFIG_" .. key)),
        configKey = Keys[key],
        min       = 0,
        max       = 60,
        step      = 5,
        integer   = true,
        help        = hi(loc("CONFIG_" .. key .. "_TT")),
        [DEPS]      = dep }; 
     end; -- size()

  local function color(key, dep)
    key = "COLOR_" .. key; 
    return { 
      inherit     = COLOR,
      title       = hi(loc("CONFIG_" .. key)),
      configKey   = Keys[key],
      help        = hi(loc("CONFIG_" .. key .. "_TT")),
      [DEPS]      = dep }; 
    end; -- color()

  local function link(key, text, instr)
    local title   = hi(loc("CONFIG_BTN_" .. key));
    local url     = loc("CONFIG_BTN_" .. key .. "_URL");
    text          = hi(loc(text  or "GET_LINK"));
    instr         = hi(loc(instr or "GET_LINK_INSTRUCTIONS"));

    return { 
      inherit     = BUTTON,
      title       = title,
      configKey   = Keys[key],
      help        = hi(loc("CONFIG_BTN_" .. key .. "_TT")),
      [DEPS]      = nil,
      text        = text,
      callback    = function() Format.popup:OpenURL(url, title, instr); end -- end of callback function                      
      }; 
    end; -- link()

    -- this creates the structure of the configuration page, which we will pass to trp3
  local CONF_STRUCT = {
    { -- [1]
    id       = "main_config_rptags_aaa_main",
    menuText = loc("APP_COLOR_NAME") .. loc("CONFIG_PAGE_MAIN"), 
    pageText = loc("APP_COLOR_NAME") .. loc("CONFIG_PAGE_MAIN"), 
    elements = { -- these are the widgets for each config option
                 -- the h1()s are just set off for source code clarity         
                     check("LOGIN_MESSAGE" ), -- second column is for dependencies
                     check("CHANGES_MESSAGE", { Keys["LOGIN_MESSAGE"] } ), 
      h1("DISPLAY"), check("IC_ONLY"),  
                     check("LINEBREAKS"),
      h1("PARSE"),   check("PARSE_HW"),
                     check("PARSE_AGE"),
                     check("PARSE_GENDER"),
                     check("ADULT_GENDERS", { Keys["PARSE_GENDER"] }),
        h1("LINKS"), link("CURSE"), 
                     link("DL"), 
                     link("WIKI"), 
                     link("DISC"), 
                     link("TWTR"),
        h1("DEMO"),  link("DEMO",   "COPY_UI",     "COPY_UI_INSTRUCTIONS"    ), 
                     link("MANUAL", "COPY_MANUAL", "COPY_MANUAL_INSTRUCTIONS"), 
      }, -- elements
    }, -- end [1]
    {  -- [2]
    id       = "main_config_rptags_bbb_formatting",
    menuText = loc("APP_COLOR_NAME") .. loc("CONFIG_PAGE_FORMATS"), 
    pageText = loc("APP_COLOR_NAME") .. loc("CONFIG_PAGE_FORMATS"), 
    elements = { -- these are the widgets for each config option
                 -- the h1()s are just set off for source code clarity         
      h1("FORMATS"), list("UNITS_HEIGHT"),
                     list("UNITS_WEIGHT"),
                     list("SIZEBUFF_FMT"),
                     list("GLANCE_DELIM"),                     
                     list("GLANCE_COLON"),
                     list("PROFILESIZE_FMT"),
      h1("SIZES"),   size("NAME_SIZE"   ),
                     size("CURR_SIZE"   ),
                     size("GLANCE_SIZE" ),
                     size("TITLE_SIZE"  ),
                     size("MISC_SIZE"   ),
      h1("WRAP"),    check("REAL_ELLIPSES"),
                     list("NAME_WRAP"   ),
                     list("CURR_WRAP"   ),
                     list("GLANCE_WRAP" ),
                     list("TITLE_WRAP"  ),
                     list("MISC_WRAP"   ),
      }, -- elements
    }, -- end [2]
    {  -- [3]
    id       = "main_config_rptags_ccc_colors",
    menuText = loc("APP_COLOR_NAME") .. loc("CONFIG_PAGE_COLORS"), 
    pageText = loc("APP_COLOR_NAME") .. loc("CONFIG_PAGE_COLORS"), 
    elements = { -- these are the widgets for each config option
                 -- the h1()s are just set off for source code clarity         
                     color("IC"),
                     color("OOC"),
                     color("NPC"),
                     color("MALE"),
                     color("FEMALE"),
                     color("NEUTER"),
                     color("THEY", { Keys["PARSE_GENDER"] }),
                     color("LESSTHAN"),
                     color("EQUALISH"),
                     color("GREATERTHAN"),
                     color("UNKNOWN"),
                     color("HILITE_1"),
                     color("HILITE_2"),
                     color("HILITE_3"),
      }, -- elements
    }, -- end [3]
  }; -- structure
  return CONF_STRUCT, true; -- true because we succeeded in creating the structure
end; -- function

-- Functions available under RPTAGS.utils.config
--
Config.setup    = setUpConfig
