-- ------------------------------------------------------------------------------
-- RP Tags for ElvUI and Total RP 3
-- by Oraibi, Moon Guard (US) server
--
-- Ora's twitter: http://twitter.com/oraibimoonguard
-- Download site: http://tinyurl.com/rptags-tukui
-- Documentation: http://tinyurl.com/rptags-wiki
--
-- ------------------------------------------------------------------------------
--
-- This work is licensed under the Creative Commons Attribution 4.0 International
-- (CC BY 4.0) license. To view a copy of the license, visit
--
--     https://creativecommons.org/licenses/by/4.0/
--
-- This license is acceptable for Free Cultural Works.
--
-- You are free to:
--
-- Share -- copy and redistribute the material in any medium or format
-- Adapt -- remix, transform, and build upon the material
--         for any purpose, even commercially.
-- 
-- The licensor cannot revoke these freedoms as long as you follow the 
-- license terms.
-- 
-- Under the following terms:
-- Attribution -- You must give appropriate credit, provide a link to the
--         license, and indicate if changes were made. You may do
--         so in any reasonable manner, but not in any way that
--         suggests the licensor endorses you or your use.
--
-- No additional restrictions -- You may not apply legal terms or
--         technological measures that legally restrict others from doing
--         anything the license permits.
--
-- Notices:
--
-- You do not have to comply with the license for elements of the material
-- in the public domain or where your use is permitted by an applicable
-- exception or limitation.
--
-- No warranties are given. The license may not give you all of the
-- permissions necessary for your intended use. For example, other rights
-- such as publicity, privacy, or moral rights may limit how you use the
-- material.
-- ------------------------------------------------------------------------------

  -- Utils-Tags: Utilities for creating tags in ElvUI
  -- list of tags at the end

local RPTAGS     = RPTAGS;
local Const      = RPTAGS.const;
local Utils      = RPTAGS.utils;
local _G         = _G;
local Core       = RPTAGS.core;

if not Core.helpdata then Core.helpdata = {}; end;
local Help = Core.helpdata;  -- if we don't have helpdata, we still need to store our helpdata, so we'll
                             -- create it, and hopefully not overwrite it!

if not Utils.tags then Utils.tags = {}; end;
local Tag = Utils.tags;

local isPlayerIC = Utils.import.trp3.isPlayerIC;
local Config     = Utils.config.get;
local nloc       = Utils.locale.newLoc;

local UI_EVENTS  = Const.MAIN_EVENT; -- these are the basic events that ElvUI needs to update the tags
local TAG_PREFIX = Const.TAG;        -- needs localization, what if roleplay isn't "rp" in some language?

      -- registers one tag, an event to wait for, and a method to invoke when found --------------------------
local function registerTag(tag, method, withLabel, label)

      local Events     = _G["ElvUF"].Tags.Events;
      local Methods    = _G["ElvUF"].Tags.Methods;
      local event      = UI_EVENTS;

      Events[tag]      = event;
      Methods[tag]     = function(u) -- this supports the IC_ONLY config setting
                           if isPlayerIC() or not Config("IC_ONLY")
                              then return method(u)
                              else return "";
                              end -- if
                         end -- function
      
      if withLabel
         then Events[tag  .. "-label"] = event;
              Methods[tag .. "-label"] = function(u) -- this supports the IC_ONLY config setting
                                           local value = method(u);
                                           if not isPlayerIC() or Config("IC_ONLY") then return ""; end;
                                           if not value or value == "" then return ""; end;
                                           return label .. ": " .. value .. "|r\n";
                                         end -- function
         end; -- if withLabel

end  

      -- wrapper that registers a tag with the rp: prefix
local function rpTag(partialTag, helpTopic, method)
      local withLabel, labelName = false, "";
      local semiTag = string.match(partialTag, "(%a+)");
      local fieldName, s = nloc("TAG/FIELD/" .. semiTag);

      if s then withLabel = true; labelName = fieldName; end; -- if parsed correctly

      Tag.register(TAG_PREFIX .. partialTag, method, withLabel, labelName);
      if helpTopic
         then Help[TAG_PREFIX .. partialTag] = helpTopic; 
         end; -- else don't register help
end;  

-- Utilities available under RPTAGS.utils.tags
--
Tag.refreshFrame = Utils.import.elvui.refreshFrame;
Tag.refreshAll   = Utils.import.elvui.refreshAll;
Tag.register     = registerTag; -- registers just any tag, doesn't set up help (but does look for -label)
Tag.rpTag        = rpTag;       -- registers a tag with the rp: prefix and adds to helpdata 
